#!/usr/bin/env bash
#
# Entry point for FIG experiments
#
# Author:  Carlos E. Budde
# Date:    16.10.2021
# License: GPLv3
#
###############################################################################
##
## Run type:  DEMO_RUN = 0  -->  full run  (~2 days in a PBS computer cluster)
##            DEMO_RUN = 1  -->  tiny demo (~6 hours in a desktop PC)
##
if [ -z ${DEMO_RUN+x} ];
then
	export DEMO_RUN=1;
else
	export DEMO_RUN=$DEMO_RUN;
fi
##
echo "Started experiments on `date`";
###############################################################################
##
##  Set up environment
##
echo "Setting up environment";
export RELSUFFIX="rel";   # suffix of IOSA models for reliability experiments
export AVASUFFIX="ava";   # suffix of IOSA models for availability experiments
export DIR_BASE="$PWD";
export DIR_MODELS="$DIR_BASE/models/IOSA";
export DIR_SCRIPTS="$DIR_BASE/scripts";
export DIR_RESULTS="$DIR_BASE/results/`date "+%Y_%m_%d-%H_%M_%S"`";
mkdir -p "$DIR_RESULTS";
echo "Results will be stored in $DIR_RESULTS";
##
###############################################################################
##
##  Run experiments
##
echo -n "Running experiments ";
if [ -z ${DEMO_RUN+x} ] || [ $DEMO_RUN -ne 1 ];
then
	echo "in FULL mode";
	. "$DIR_SCRIPTS"/run_full.sh;
else
	echo "in DEMO mode";
	. "$DIR_SCRIPTS"/run_demo.sh;
fi
echo "All experiments finished running";
##
###############################################################################
##
##  Plot results
##
echo "Summarising and plotting results"
. "$DIR_SCRIPTS"/plot_last_results.sh;
##
###############################################################################
echo "Finished experiments on `date`";
##
exit 0;
