#!/usr/bin/env bash
#
# Author:  Carlos E. Budde
# Date:    22.03.2016
# License: GPLv3
#
#   NOTE: this script should be sourced from another script
#   Cool site: http://wiki.bash-hackers.org


# Check argument is a regular file
is_file() {
	if [ ! -f "$1" ]; then
		echo "[ERROR] No file named \"$1\"" 1>&2;
		return 1;
	#else
	#	echo "File \"$1\" found";
	fi
}

# Common pattern of FIG execution command
#  - arg1: timeout (in seconds)
#  - arg2: number of repetitions
execution() {
	local REPETITIONS=$(printf " --stop-time $1%.0s" `seq $2`);
	echo --parlare 1 --ft $REPETITIONS;
}

# Print simulation name and start time
timestamp() {
	echo -n "  - Running $1: "; date "+%Y.%m.%d @ %H:%M:%S";
}

# Extract CI precision values from file (for requested confidence level)
# If no value is found returns 0.0
#  - arg1: confidence level, e.g. "95%"
#  - arg2: file to parse
extract_precision() {
	(grep "$1" --after=1 "$2" || echo " - precision: 0.0") |
		awk '/precision:/ { print $3 }';
}

# Poll until less than MAXJOBSN jobs called 'fig' are running
poll_till_free() {
	if [ -z $MAXJOBSN ] || [ $MAXJOBSN -le 0 ]; then
		local JOBSBOUND=`nproc --all`;
	else
		local JOBSBOUND=$MAXJOBSN;
	fi
	# do-while syntax sugar for bash
	while
		#local RUNNING=`ps -fC "fig" | grep "\<$1" | wc -l`
		local RUNNING=`ps -fC "fig" | grep "fig" | wc -l`
		[ $RUNNING -ge $JOBSBOUND ]
	do
		sleep 10s;
	done
	sleep 0.1;  # give previous job a little while to sink in
}

# Wait until the number of jobs running in PBS is below the threshold
#  - arg1: per-user job limit to respect
#  - arg2: job name pattern, to filter out qstat header and other garbage
wait_for_PBS_queue() {
	while
		local RUNNING=`qstat -u "$(whoami)" 2>/dev/null | grep "$2" | wc -l 2>/dev/null`
		[ $RUNNING -ge $1 ]
	do
		sleep 30;
	done
}


return 0

