#!/usr/bin/env bash
#
# Bash script for reliability analysis (transient properties)
#
# Author:  Carlos E. Budde
# Date:    21.10.2021
# License: GPLv3
#
#   NOTE: this script should be sourced from another script
#   Cool site: http://wiki.bash-hackers.org

source "$DIR_SCRIPTS/fig_utils.sh";

# Rare event simulation algorithms to try
declare -A ALGOS_REL;    # XXX reflect changes to this array in plots.cfg
ALGOS_REL[RES_ES]="$EXEC $MODEL -e restart -t es        --acomp";
ALGOS_REL[RES_2]=" $EXEC $MODEL -e restart -t hyb -g  2 --acomp";
ALGOS_REL[FE_ES]=" $EXEC $MODEL -e sfe     -t es        --acomp";
ALGOS_REL[FE_8]="  $EXEC $MODEL -e sfe     -t hyb -g  8 --acomp";

# Importance functions to try
declare -A IFUNS;    # XXX reflect changes to this array in plots.cfg
IFUNS[STR]="` tail ${MODEL} | awk '/IFUN STRUCTURAL:/ { print $4 }'`";
IFUNS[MCS]="` tail ${MODEL} | awk '/IFUN MCS:/        { print $4 }'`";

# Run experiments:
echo "Starting experiment $JOB";

# - first His Lordship crude Monte Carlo
poll_till_free;
SIM="CMC";
LOG="$DIR_RESULTS/${JOB}_$SIM";
fig $EXEC "$MODEL" --cmc 1> "$LOG.out" 2> "$LOG.err" &
timestamp $SIM;

# - then each rare event simulation algorithm...
for A in "${!ALGOS_REL[@]}"; do
	# ...with each importance function
	for I in "${!IFUNS[@]}"; do
		poll_till_free;
		SIM="${A}_$I";
		LOG="$DIR_RESULTS/${JOB}_$SIM";
		fig ${ALGOS_REL[$A]} ${IFUNS[$I]} 1> "$LOG.out" 2> "$LOG.err" &
		timestamp $SIM;
	done
done

echo "Launched all runs for experiment $JOB";

return 0;
