#!/usr/bin/env gnuplot
#
# Gnuplot script to plot CI precision values from FIG experimentation
#
# Author:  Carlos E. Budde
# Date:    23.10.2021
# License: GPLv3
#
remove(x,s)=(i0=strstrt(s,x),i0?remove(x,s[:i0-1].s[i0+strlen(x):]):s)
replace(x,y,s)=(i0=strstrt(s,x),i0?(replace(x,y,s[:i0-1].y.s[i0+strlen(x):])):s)
# ^^^ doing this correctly (with two recursive calls to replace) gives stack overflow
YMIN=system("echo $PMIN")
YMAX=system("echo $PMAX")
FULLRUN=system("if [ -z ${DEMO_RUN+x} ] || [ $DEMO_RUN -ne 1 ]; then echo 1; else echo 0; fi")
EXPERIMENT=system("echo $EXPERIMENT")
# Importance functions
IFUN0=system("echo $IFUN_0")
IFUN1=system("echo $IFUN_1")
IFUN2=system("echo $IFUN_2")
# Data files
DAT0=system("echo $DAT_0")
DAT1=system("echo $DAT_1")
DAT2=system("echo $DAT_2")
DAT3=system("echo $DAT_3")
DAT4=system("echo $DAT_4")
DAT5=system("echo $DAT_5")
DAT6=system("echo $DAT_6")
# Functions names
FUN0=replace("_","-",system("echo $FUN_0"))
FUN1=replace("_","-",system("echo $FUN_1"))
FUN2=replace("_","-",system("echo $FUN_2"))
FUN3=replace("_","-",system("echo $FUN_3"))
FUN4=replace("_","-",system("echo $FUN_4"))
FUN5=replace("_","-",system("echo $FUN_5"))
FUN6=replace("_","-",system("echo $FUN_6"))
# Bars colours
COL0=system("echo $COL_0")
COL1=system("echo $COL_1")
COL2=system("echo $COL_2")
COL3=system("echo $COL_3")
COL4=system("echo $COL_4")
COL5=system("echo $COL_5")
COL6=system("echo $COL_6")
#
set terminal svg size 824,360 enhanced font 'Latin Modern Sans,32'
set out EXPERIMENT.".svg"
#
unset title
set style data histogram
set style histogram errorbars gap 2
set style fill solid 1.0
set style line  1 lt 1 lw 0 lc rgb COL0
set style line  2 lt 1 lw 0 lc rgb COL1
set style line  3 lt 1 lw 0 lc rgb COL2
set style line  4 lt 1 lw 0 lc rgb COL3
set style line  5 lt 1 lw 0 lc rgb COL4
if (FULLRUN) {
	set style line  6 lt 1 lw 0 lc rgb COL5
	set style line  7 lt 1 lw 0 lc rgb COL6
}
set yrange [YMIN:YMAX]
set grid ytics lw .9 #dt 3 
########################################################
# y-axis labels formatting:    #########################
# - lin-scale                  #########################
set format y "%2.1te{%T}"
RPOS=(YMIN-.08*(YMAX-YMIN))
CPOS=(YMIN+.05*(YMAX-YMIN))
# - log-scale                  #########################
#set log y
#set format y "10^{%T}"  # like LaTeX $10^{\bf -4}$ 
#RPOS=0.2*YMIN
#CPOS=3.5*YMIN
########################################################
set ytics font ",18" scale 0.5 right offset .6,0 nomirror
unset xtics
unset xlabel
#unset key
set lmargin 3.5
set bmargin -.5
#
if (FULLRUN) {
	set offsets -.6,-.4,-.2,10
	set bars .5 lc -1 lw 1.5
	set key font ",13"
	set key maxrows 1 maxcols 11  samplen .1 spacing .2 at screen .92,.93
	set label "CMC" font ",20" at -.12,(RPOS) front
	set label IFUN0 font ",20" at 0.69,(RPOS) front
	set label IFUN1 font ",20" at 1.68,(RPOS) front
	set label IFUN2 font ",20" at 2.64,(RPOS) front
	plot \
	newhistogram "", \
	DAT0 u 3:4 ls 1 t FUN0, \
	DAT0 u (0):(CPOS):5 w labels font "Sans:Bold,14" tc rgb "gray80" t "" ,\
	newhistogram "" at .8, \
	DAT1 u 3:4 ls 2 t FUN1, \
	DAT2 u 3:4 ls 3 t FUN2, \
	DAT3 u 3:4 ls 4 t FUN3, \
	DAT4 u 3:4 ls 5 t FUN4, \
	DAT5 u 3:4 ls 6 t FUN5, \
	DAT6 u 3:4 ls 7 t FUN6, \
	DAT1 u ($1-.520):(CPOS):5 w labels font "Sans:Bold,14" tc rgb "black"  t "", \
	DAT2 u ($1-.385):(CPOS):5 w labels font "Sans:Bold,14" tc rgb "black"  t "", \
	DAT3 u ($1-.265):(CPOS):5 w labels font "Sans:Bold,14" tc rgb "gray80" t "", \
	DAT4 u ($1-.136):(CPOS):5 w labels font "Sans:Bold,14" tc rgb "black"  t "", \
	DAT5 u ($1-.010):(CPOS):5 w labels font "Sans:Bold,14" tc rgb "black"  t "", \
	DAT6 u ($1+.115):(CPOS):5 w labels font "Sans:Bold,14" tc rgb "gray80" t "", \
} else {
	set bars .7 lc -1 lw 2
	set key font ",16"
	set key horizontal samplen .3 at screen .8,.97
	set label "CMC" font ",22" at -.2,(RPOS) front
	set label IFUN0 font ",22" at 0.7,(RPOS) front
	set label IFUN1 font ",22" at 1.7,(RPOS) front
	plot \
	newhistogram "", \
	DAT0 u 3:4 ls 1 t FUN0, \
	DAT0 u (0):(CPOS):5 w labels font "Sans:Bold,26" tc rgb "gray80" t "" ,\
	newhistogram "" at .8, \
	DAT1 u 3:4 ls 2 t FUN1, \
	DAT2 u 3:4 ls 3 t FUN2, \
	DAT3 u 3:4 ls 4 t FUN3, \
	DAT4 u 3:4 ls 5 t FUN4, \
	DAT1 u ($1-.450):(CPOS):5 w labels font "Sans:Bold,26" tc rgb "black"  t "", \
	DAT2 u ($1-.280):(CPOS):5 w labels font "Sans:Bold,26" tc rgb "gray80" t "", \
	DAT3 u ($1-.115):(CPOS):5 w labels font "Sans:Bold,26" tc rgb "black"  t "", \
	DAT4 u ($1+.060):(CPOS):5 w labels font "Sans:Bold,26" tc rgb "gray80" t "", \
}


# Lines "" u ($1+.390):(1.5*YMIN):5 w labels ... place the white CI counts
#              ^^^^^^^ this is the horizontal alignment, to match the proper bar
