#!/usr/bin/env bash
#
# Small experimental demonstration (single machine)
#
# Author:  Carlos E. Budde
# Date:    21.10.2021
# License: GPLv3
#
#   NOTE: this script should be sourced from another script
#   Cool site: http://wiki.bash-hackers.org

source "$DIR_SCRIPTS/fig_utils.sh";

export NUM_REPS=4;  # number of times each experiment is repeated

# Experiments for transient analysis
declare -A JOBS_REL;  # XXX plot_last_results.sh reads this array !!! XXX
# XXX The keys of this array must match prefixes of IOSA models for reliability
JOBS_REL[DSPARE_3]=300;   #    5 m
JOBS_REL[HECS_2]=20;      #   20 s
JOBS_REL[HECS_3]=300;     #    5 m
JOBS_REL[FTPP_4]=30;      #   30 s
JOBS_REL[FTPP_5]=240;     #    4 m
JOBS_REL[HVC_4]=90;       #   90 s
JOBS_REL[HVC_5]=300;      #    5 m

# Experiments for steady-state analysis
declare -A JOBS_AVA;  # XXX plot_last_results.sh reads this array !!! XXX
# XXX The keys of this array must match prefixes of IOSA models for availability
JOBS_AVA[HECS_1]=5;       #    5 s
JOBS_AVA[HECS_2]=20;      #   20 s
JOBS_AVA[HECS_3]=120;     #    2 m
JOBS_AVA[RWC_1]=30;       #   30 s
JOBS_AVA[RWC_2]=300;      #    5 m
JOBS_AVA[RC_3]=30;        #   30 s
JOBS_AVA[RC_4]=300;       #    5 m

# Launch each reliability experiment in sequence
echo "Launching experiments to study unreliability (transient properties)";
for J in "${!JOBS_REL[@]}"; do
	export JOB="${J}_$RELSUFFIX";
	export TIME=${JOBS_REL[$J]};  # time that each individual simulation runs
	export EXEC="`execution $TIME $NUM_REPS`";
	export MODEL="$DIR_MODELS/${JOB}.sa";
	is_file "$MODEL";
	source "$DIR_SCRIPTS"/job_rel.sh 1> "$DIR_RESULTS/$JOB.out" \
	                                 2> "$DIR_RESULTS/$JOB.err";
	echo "  - Running all experiment for $JOB";
	sleep 1;  # ... just...(pant) gimme a sec...(pant) phew! let's go
done

# Launch each availability experiment in sequence
echo "Launching experiments to study unavailability (steady-state properties)";
for J in "${!JOBS_AVA[@]}"; do
	export JOB="${J}_$AVASUFFIX";
	export TIME=${JOBS_AVA[$J]};  # time that each individual simulation runs
	export EXEC="`execution $TIME $NUM_REPS`";
	export MODEL="$DIR_MODELS/${JOB}.sa";
	is_file "$MODEL";
	source "$DIR_SCRIPTS"/job_ava.sh 1> "$DIR_RESULTS/$JOB.out" \
	                                 2> "$DIR_RESULTS/$JOB.err";
	echo "  - Running all experiment for $JOB";
	sleep 1;  # ... just...(pant) gimme a sec...(pant) phew! let's go
done

# Wait for all jobs
echo "Waiting for all experiments to finish";
while
	RUNNING=`ps -fC "fig" | grep "fig" | wc -l`
	[ $RUNNING -gt 0 ]
do
	sleep 10s;
done

return 0;

