#!/usr/bin/env sh
#
# Artifact entry point
#
# Author:  Carlos E. Budde
# Date:    04.11.2021
# License: GPLv3
#
# Set: DEMO_RUN = 1  -->  minimal demo (~6 hours in a desktop PC)
#      DEMO_RUN = 0  -->  full experimental reproduction (~2 days in PBS cluster)
export DEMO_RUN=1;
#
echo;
echo "Artifact for experimental reproduction of the paper:";
echo "\"Analysis of non-Markovian repairable fault trees through rare event simulation\"";
echo "by C.E. Budde, P.R. D'Argenio, R.E. Monti, and M. Stoelinga.";
echo;
sleep 1;
#
set -e;
cd ./setup;
#
echo "--[ Preparing environment ]-----------------------------------";
if ! type gnuplot 2>&1 >/dev/null; then
	echo "Installing artifact dependencies (bison, numpy, gnuplot, etc.)";
	sh install_dependencies.sh;
	type gnuplot >/dev/null || (echo "Installation failed; aborting" && exit 1);
fi
if ! type fig 2>&1 >/dev/null; then
	echo "Installing FIG";
	sh install_fig.sh;
	type fig >/dev/null || (echo "Installation failed; aborting" && exit 1);
fi
echo "--[ Done ]----------------------------------------------------";
echo;
#
cd ../experiments;
#
set +e;
echo "--[ Starting experimental reproduction ]----------------------";
bash run.sh 2>&1 | tee run.log;
echo "Run log saved in $PWD/run.log";
echo "--[ Done ]----------------------------------------------------";
echo;
#
cd ..;
#
exit 0;
