" Vim syntax file
" Language:		Kepler syntax for repairable dynamic fault trees
" Maintainer:		Carlos E. Budde  <cbudde@famaf.unc.edu.ar>
" Last Change:		2021 Oct 18
" Original Author:	Carlos E. Budde  <cbudde@famaf.unc.edu.ar>
"
" Generated from VIM's haskell syntax highlight file
"
" Options-assign a value to these variables to turn the option on:
"
" hs_highlight_delimiters - Highlight delimiter characters--users
"			    with a light-colored background will
"			    probably want to turn this on.
" hs_highlight_boolean - Treat True and False as keywords.
" hs_highlight_types - Treat names of primitive types as keywords.
" hs_highlight_more_types - Treat names of other common types as keywords.
" hs_highlight_debug - Highlight names of debugging functions.
" hs_allow_hash_operator - Don't highlight seemingly incorrect C
"			   preprocessor directives but assume them to be
"			   operators

" quit when a syntax file was already loaded
if exists("b:current_syntax")
  finish
endif

" Show primitive types as keywords
let hs_highlight_types = 1
let hs_highlight_debug = 1

" Highlight booleans
let hs_highlight_boolean = 1

" (Qualified) identifiers (no default highlighting)
syn match ConId "\(\<[A-Z][a-zA-Z0-9_']*\.\)\=\<[A-Z][a-zA-Z0-9_']*\>"
syn match VarId "\(\<[A-Z][a-zA-Z0-9_']*\.\)\=\<[a-z][a-zA-Z0-9_']*\>"

" Infix operators--most punctuation characters and any (qualified) identifier
" enclosed in `backquotes`. An operator starting with : is a constructor,
" others are variables (e.g. functions).
syn match hsVarSym "\(\<[A-Z][a-zA-Z0-9_']*\.\)\=[-!#$%&\*\+/<=>\?@\\^|~.][-!#$%&\*\+/<=>\?@\\^|~:.]*"
syn match hsConSym "\(\<[A-Z][a-zA-Z0-9_']*\.\)\=:[-!#$%&\*\+./<=>\?@\\^|~:]*"
syn match hsVarSym "`\(\<[A-Z][a-zA-Z0-9_']*\.\)\=[a-z][a-zA-Z0-9_']*`"
syn match hsConSym "`\(\<[A-Z][a-zA-Z0-9_']*\.\)\=[A-Z][a-zA-Z0-9_']*`"

" Reserved symbols--cannot be overloaded.
syn match hsDelimiter  "(\|)\|\[\|\]\|,\|;\|_\|{\|}"

" Strings and constants
syn match   hsSpecialChar	contained "\\\([0-9]\+\|o[0-7]\+\|x[0-9a-fA-F]\+\|[\"\\'&\\abfnrtv]\|^[A-Z^_\[\\\]]\)"
syn match   hsSpecialChar	contained "\\\(NUL\|SOH\|STX\|ETX\|EOT\|ENQ\|ACK\|BEL\|BS\|HT\|LF\|VT\|FF\|CR\|SO\|SI\|DLE\|DC1\|DC2\|DC3\|DC4\|NAK\|SYN\|ETB\|CAN\|EM\|SUB\|ESC\|FS\|GS\|RS\|US\|SP\|DEL\)"
syn match   hsSpecialCharError	contained "\\&\|'''\+"
syn region  hsString		start=+"+  skip=+\\\\\|\\"+  end=+"+  contains=hsSpecialChar
syn match   hsCharacter		"[^a-zA-Z0-9_']'\([^\\]\|\\[^']\+\|\\'\)'"lc=1 contains=hsSpecialChar,hsSpecialCharError
syn match   hsCharacter		"^'\([^\\]\|\\[^']\+\|\\'\)'" contains=hsSpecialChar,hsSpecialCharError
syn match   hsNumber		"[0-9]\+\>\|\<0[xX][0-9a-fA-F]\+\>\|\<0[oO][0-7]\+"
syn match   keplerFloat		"\(\.[0-9]\+\)\|\([0-9]\+\(\.[0-9]\+\)\?\([eE][-+]\=[0-9]\+\)\=\)"

" Keyword definitions. These must be patters instead of keywords
" because otherwise they would match as keywords at the start of a
" "literate" comment (see lhs.vim).
syn match keplerTLE		"\<toplevel\>"
syn match keplerOR		"\<or\>"
syn match keplerAND		"\<and\>"
syn match keplerVOT		"\<[0-9]\+of[0-9]\+\>"
syn match keplerPAND		"\<pand\>"
syn match keplerFDEP		"\<fdep\>"
syn match keplerSPARE		"\<spare\>"
syn match keplerRBOX		"\<rbox\>"
syn match keplerFAIL		"\<fail"
syn match keplerDORM		"\<dorm"
syn match keplerREPAIR		"\<repair"
"syn match hsStatement		"\<\(do\|case\|of\|let\|in\)\>"
"syn match hsConditional		"\<\(if\|then\|else\)\>"

" Not real keywords, but close.
if exists("hs_highlight_boolean")
  " Boolean constants from the standard prelude.
  "syn match keplerBoolean "\<\(true\|false\)\>"
  syn match keplerRepairPolicy "\<\(prio\|fcfs\|rand\)\>"
  syn match keplerIsDistributed "\~"
endif
if exists("hs_highlight_types")
  " Primitive types from the standard prelude and libraries.
  syn match keplerType "\(exponential\|hyperexponential2\|uniform\|lognormal\|normal\|erlang\|rayleigh\|gamma\|weibull\|dirac\)"
endif
if exists("hs_highlight_more_types")
  " Types from the standard prelude libraries.
  syn match hsExitCode "\<\(ExitSuccess\)\>"
endif
if exists("hs_highlight_debug")
  " Debugging functions from the standard prelude.
  "syn match keplerExtra "\<\(const\)\>"
endif


" Comments
syn match   hsLineComment      "---*\([^-!#$%&\*\+./<=>\?@\\^|~].*\)\?$" contains=keplerLineComment
syn match   keplerLineComment    "///*\([^-!#$%&\*\+./<=>\?@\\^|~].*\)\?$" contains=hsLineComment
syn region  hsBlockComment     start="{-"  end="-}" contains=hsBlockComment
syn region  hsPragma	       start="{-#" end="#-}"

" C Preprocessor directives. Shamelessly ripped from c.vim and trimmed
" First, see whether to flag directive-like lines or not
if (!exists("hs_allow_hash_operator"))
    syn match	cError		display "^\s*\(%:\|#\).*$"
endif
" Accept %: for # (C99)
syn region	cPreCondit	start="^\s*\(%:\|#\)\s*\(if\|ifdef\|ifndef\|elif\)\>" skip="\\$" end="$" end="//"me=s-1 contains=cComment,cCppString,cCommentError
syn match	cPreCondit	display "^\s*\(%:\|#\)\s*\(else\|endif\)\>"
syn region	cCppOut		start="^\s*\(%:\|#\)\s*if\s\+0\+\>" end=".\@=\|$" contains=cCppOut2
syn region	cCppOut2	contained start="0" end="^\s*\(%:\|#\)\s*\(endif\>\|else\>\|elif\>\)" contains=cCppSkip
syn region	cCppSkip	contained start="^\s*\(%:\|#\)\s*\(if\>\|ifdef\>\|ifndef\>\)" skip="\\$" end="^\s*\(%:\|#\)\s*endif\>" contains=cCppSkip
syn region	cIncluded	display contained start=+"+ skip=+\\\\\|\\"+ end=+"+
syn match	cIncluded	display contained "<[^>]*>"
syn match	cInclude	display "^\s*\(%:\|#\)\s*include\>\s*["<]" contains=cIncluded
syn cluster	cPreProcGroup	contains=cPreCondit,cIncluded,cInclude,cDefine,cCppOut,cCppOut2,cCppSkip,cCommentStartError
syn region	cDefine		matchgroup=cPreCondit start="^\s*\(%:\|#\)\s*\(define\|undef\)\>" skip="\\$" end="$"
syn region	cPreProc	matchgroup=cPreCondit start="^\s*\(%:\|#\)\s*\(pragma\>\|line\>\|warning\>\|warn\>\|error\>\)" skip="\\$" end="$" keepend

syn region	cComment	matchgroup=cCommentStart start="/\*" end="\*/" contains=cCommentStartError,cSpaceError contained
syntax match	cCommentError	display "\*/" contained
syntax match	cCommentStartError display "/\*"me=e-1 contained
syn region	cCppString	start=+L\="+ skip=+\\\\\|\\"\|\\$+ excludenl end=+"+ end='$' contains=cSpecial contained

" Define the default highlighting.
" Only when an item doesn't have highlighting yet

"hi def link keplerOR		Structure
hi def link keplerTLE		Conditional
hi def link keplerOR		Conditional
hi def link keplerAND		Conditional
hi def link keplerVOT		Conditional
hi def link keplerPAND		Conditional
hi def link keplerFDEP		Conditional
hi def link keplerSPARE		Conditional
hi def link keplerRBOX		Conditional
hi def link keplerFAIL		Conditional
hi def link keplerDORM		Conditional
hi def link keplerREPAIR	Conditional
"hi def link hsStatement			  Statement
hi def link hsConditional			  Conditional
hi def link hsSpecialChar			  SpecialChar
hi def link hsTypedef			  Typedef
hi def link hsVarSym			  hsOperator
hi def link hsConSym			  hsOperator
hi def link hsOperator			  Operator
if exists("hs_highlight_delimiters")
" Some people find this highlighting distracting.
hi def link hsDelimiter			  Delimiter
endif
hi def link hsSpecialCharError		  Error
hi def link hsString			  String
hi def link hsCharacter			  Character
hi def link hsNumber			  Number
hi def link keplerFloat			  Float
hi def link hsConditional		  Conditional
hi def link hsLiterateComment		  hsComment
hi def link hsBlockComment		  hsComment
hi def link hsLineComment		  hsComment
hi def link keplerLineComment		  hsComment
hi def link hsComment			  Comment
hi def link hsPragma			  SpecialComment
"hi def link keplerBoolean		  Boolean
hi def link keplerRepairPolicy		  Boolean
hi def link keplerIsDistributed		  Boolean
hi def link keplerType			  Type
hi def link hsEnumConst			  Constant
"hi def link keplerExtra			  Debug

hi def link cCppString		hsString
hi def link cCommentStart		hsComment
hi def link cCommentError		hsError
hi def link cCommentStartError	hsError
hi def link cInclude		Include
hi def link cPreProc		PreProc
hi def link cDefine		Macro
hi def link cIncluded		hsString
hi def link cError			Error
hi def link cPreCondit		PreCondit
hi def link cComment		Comment
hi def link cCppSkip		cCppOut
hi def link cCppOut2		cCppOut
hi def link cCppOut		Comment


let b:current_syntax = "kepler"

" Options for vi: ts=8 sw=2 sts=2 nowrap noexpandtab ft=vim

